/*
 * The SmartWeb Framework
 * Copyright (C) 2004-2006
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * For further informations on the SmartWeb Framework please visit
 *
 *                        http://smartweb.sourceforge.net
 */
package net.smartlab.web.test;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.net.URL;
import java.sql.Connection;
import java.sql.DriverManager;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.dbunit.DatabaseTestCase;
import org.dbunit.database.DatabaseConfig;
import org.dbunit.database.DatabaseConnection;
import org.dbunit.database.IDatabaseConnection;
import org.dbunit.database.QueryDataSet;
import org.dbunit.dataset.DefaultDataSet;
import org.dbunit.dataset.IDataSet;
import org.dbunit.dataset.xml.FlatXmlDataSet;
import org.dbunit.operation.DatabaseOperation;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.Configuration;
import net.smartlab.web.BusinessObjectFactory;
import net.smartlab.web.config.ConfigurationStrategy;

/**
 * Base class for BusinessObjectFactory testing.
 * 
 * @author rlogiacco
 */
public abstract class BusinessObjectFactoryTestCase extends DatabaseTestCase {

	/**
	 * Provides logging for the test cases.
	 */
	protected Log logger = LogFactory.getLog(this.getClass());

	/**
	 * The connection to the database in the dbUnit style.
	 */
	private DatabaseConnection connection;


	/**
	 * Default constructor.
	 */
	public BusinessObjectFactoryTestCase() {
		super();
	}

	/**
	 * Named constructor.
	 * 
	 * @param name test name.
	 */
	public BusinessObjectFactoryTestCase(String name) {
		super(name);
	}

	/**
	 * @see junit.framework.TestCase#setUp()
	 */
	protected void setUp() throws Exception {
		if (logger.isDebugEnabled()) {
			logger.debug("setUp() - start");
		}
		BusinessObjectFactory.setConfigurationStrategy(new TestConfiguration());
		super.setUp();
	}

	/**
	 * @see junit.framework.TestCase#tearDown()
	 */
	protected void tearDown() throws Exception {
		if (logger.isDebugEnabled()) {
			logger.debug("tearDown() - start");
		}
		BusinessObjectFactory.close();
		super.tearDown();
	}

	/**
	 * Returns the configuration file to be used for test purposes. The default
	 * implementation uses <code>res/smartweb-test.hcf</code> but the method
	 * could be overriden to support different configuration files for different
	 * tests.
	 * 
	 * @return the configuration file to be used for test purposes.
	 * @throws Exception if the file open fails.
	 */
	protected URL getConfigurationFile() throws Exception {
		return new File("res/hibernate.cfg.xml").toURL();
	}


	/**
	 * TODO documentation
	 * @author rlogiacco
	 */
	protected class TestConfiguration implements ConfigurationStrategy {

		private SessionFactory factory = null;


		/**
		 * @see net.smartlab.web.config.ConfigurationStrategy#getSessionFactory(net.smartlab.web.BusinessObjectFactory)
		 */
		public SessionFactory getSessionFactory(BusinessObjectFactory bof) {
			if (factory == null) {
				try {
					Configuration configuration = new Configuration().configure(getConfigurationFile());
					String driver = configuration.getProperty("hibernate.connection.driver_class");
					String url = configuration.getProperty("hibernate.connection.url");
					String username = configuration.getProperty("hibernate.connection.username");
					String password = configuration.getProperty("hibernate.connection.password");
					Class.forName(driver);
					connection = new DatabaseConnection(DriverManager.getConnection(url, username, password));
					factory = configuration.buildSessionFactory();
				} catch (Exception e) {
					logger.error(e);
				}
			}
			return factory;
		}
	}


	/**
	 * @see org.dbunit.DatabaseTestCase#getConnection()
	 */
	protected IDatabaseConnection getConnection() throws Exception {
		if (logger.isDebugEnabled()) {
			logger.debug("getConnection() - start");
		}
		if (connection == null) {
			Configuration configuration = new Configuration().configure(getConfigurationFile());
			String driver = configuration.getProperty("hibernate.connection.driver_class");
			String url = configuration.getProperty("hibernate.connection.url");
			String username = configuration.getProperty("hibernate.connection.username");
			String password = configuration.getProperty("hibernate.connection.password");
			Class.forName(driver);
			Connection c = DriverManager.getConnection(url, username, password);
			connection = new DatabaseConnection(c);
			connection.getConfig().setFeature(DatabaseConfig.FEATURE_QUALIFIED_TABLE_NAMES, true);
			connection.getConfig().setProperty(DatabaseConfig.PROPERTY_ESCAPE_PATTERN, "\"?\"");
		}
		return connection;
	}

	/**
	 * @see org.dbunit.DatabaseTestCase#getDataSet()
	 */
	protected IDataSet getDataSet() throws Exception {
		if (logger.isDebugEnabled()) {
			logger.debug("getDataSet() - start");
			logger.trace("  res = " + this.getDataSetFile());
		}
		try {
			return new FlatXmlDataSet(new FileInputStream(this.getDataSetFile()));
		} catch (Exception e) {
			logger.warn("getDataSet() - not found");
			return new DefaultDataSet();
		}
	}

	/**
	 * @see org.dbunit.DatabaseTestCase#getSetUpOperation()
	 */
	protected DatabaseOperation getSetUpOperation() throws Exception {
		return DatabaseOperation.REFRESH;
	}

	/**
	 * @see org.dbunit.DatabaseTestCase#getTearDownOperation()
	 */
	protected DatabaseOperation getTearDownOperation() throws Exception {
		return DatabaseOperation.NONE;
	}

	/**
	 * Method implementations should return the dataset file name.
	 * 
	 * @return the dataset file name.
	 */
	protected abstract String getDataSetFile();

	/**
	 * Partial dataset export allows to select which tables to include into the
	 * dataset.
	 * 
	 * @param tables a two dimensions String array
	 * @throws Exception if something wrong occurs during dataset creation.
	 */
	public void export(String[][] tables) throws Exception {
		// partial database export
		QueryDataSet dataSet = new QueryDataSet(this.getConnection());
		for (int i = 0; i < tables.length; i++) {
			if (tables[i][1] == null) {
				dataSet.addTable(tables[i][0]);
			} else {
				dataSet.addTable(tables[i][0], tables[i][1]);
			}
		}
		FlatXmlDataSet.write(dataSet, new FileOutputStream(this.getDataSetFile()));
	}
}
